fhir is a work in progress, tracking todo items at https://github.com/PotatoEMR/gleam-fhir/issues, if anything is incorrect, missing, hard to use, or poorly documented, please add an issue.

0.6.0 May 6, 2026

0.5.0 April 18, 2026

0.4.0 April 12, 2026

0.3.0 April 11, 2026

0.2.0 Mar 15, 2026

0.1.0 Feb 07, 2026

0.0.0 Feb 01, 2026

Search Document