0.1.0 Feb 07, 2026

0.0.0 Feb 01, 2026

(once on 1.0.0) Version number MAJOR.MINOR.PATCH means:

MAJOR - incompatible API changes

MINOR - backward-compatible functionality added

PATCH - backward-compatible bug fixes

Gleam fhir is a work in progress, tracking todo items at https://github.com/PotatoEMR/gleam-fhir/issues, if anything is incorrect, missing, or hard to use, please add an issue.

Search Document