Primitive Type

FHIR primitive types are a single value. In Gleam, primitive types are record fields with a single value, rather than with child fields. For example, the Patient resource has a Patient.active boolean element, which in Gleam is a Bool in r4.Patient.

Patient.active

pub type Patient {
  Patient(
    ...
    active: Option(Bool),
    ...
  )
}
Primitive TypeIn Gleam
base64BinaryString
booleanBool
canonicalString
codeenum if required binding, otherwise String
dateString
dateTimeString
decimalFloat
idString
instantString
integerInt
integer64String
markdownString
oidString
stringString
positiveIntInt
timeString
unsignedIntInt
uriString
urlString
uuidString
Search Document